#!/usr/bin/env python3
""" Module handling ublox modems """

import re
import time
from ..wwan_modem import WwanModem
from ...logger import logger
from ...exceptions import UnexpectedFirmwareVersion

class UbloxWwanModem(WwanModem):
    """ Ublox modem definition """

    # Placeholder, will mark all undefined ublox modems as untested
    LATEST_FW = "00.00,A00.00"
    RECOMMENDED_FW = "00.00,A00.00"

    def __init__(self, modem):
        # Don't call super init
        # Instead copy state of already initialized modem
        self.__dict__.update(modem.__dict__)

    def reset(self):
        """ Reset the modem """
        logger.info("Resetting modem")
        try:
            self.send_command(b'AT+CFUN=16', no_answer=True)
        except Exception:
            pass  # When resetting, the modem does not answer

        # Wait until modem reappears
        self.close_serial()
        self.wait_on_down()
        self.wait_on_up()
        self.open_serial()

    def reload_sim(self):
        """ Reload sim card when it has been switched """
        logger.info("Reloading sim card")
        self.send_command(b'AT+CFUN=19', no_answer=True)
        time.sleep(2)
        self.send_command(b'AT+CFUN=1', no_answer=True)
        time.sleep(2)

    @staticmethod
    def _firmware_strtoi(fw_str):
        """ Convert firmware string to comparable integer """
        # Example engineering firmware: 17.00_ENG0702,A01.00
        # Example standard (recent) firmware: 17.00,A01.01
        # Example standard (old) firmware: 16.19
        if ',A' in fw_str:  # Separation between firmware and app versions
            regexed = re.search(r"(\d{2})\.(\d{2})(?:_ENG\d{4})?,A(\d{2})\.(\d{2})", fw_str)
            try:
                fw_str = ''.join(regexed.group(1,2,3,4))
            except IndexError as exc:
                raise UnexpectedFirmwareVersion() from exc
        else:
            fw_str += '0000'  # Default App version
            fw_str = fw_str.replace('.','')

        try:
            return int(fw_str)
        except ValueError as exc:
            raise UnexpectedFirmwareVersion() from exc

    def _get_firmware_number(self):
        """ Returns the firmware as comparable int """
        fw_str = super().get_firmware_version()
        return self._firmware_strtoi(fw_str)

    def get_firmware(self):
        """ Returns the fw as string and an indication of its "quality" """
        fw_str = super().get_firmware_version()

        try:
            fw_nbr = self._firmware_strtoi(fw_str)
        except UnexpectedFirmwareVersion:
            return fw_str, WwanModem.UNTESTED_FW

        rec_fw_nbr = self._firmware_strtoi(self.RECOMMENDED_FW)
        last_fw_nbr = self._firmware_strtoi(self.LATEST_FW)

        if fw_nbr > last_fw_nbr:
            return fw_str, WwanModem.UNTESTED_FW

        if fw_nbr == last_fw_nbr:
            return fw_str, WwanModem.LATEST_FW

        if fw_nbr >= rec_fw_nbr:
            return fw_str, WwanModem.DEP_FW

        return fw_str, WwanModem.OBS_FW
