#!/usr/bin/env python3
""" Module handling ublox TOBY-L2  modems """

import re
from .base import UbloxWwanModem
from ...logger import logger
from ...exceptions import AtCommandException

class UbloxLaraWwanModem(UbloxWwanModem):
    """ LARA-L6 type of modem """

    LATEST_FW = "03.15,A00.01"
    RECOMMENDED_FW = "03.15,A00.01"

    def __init__(self, modem):
        UbloxWwanModem.__init__(self, modem)

    def get_vendor_configs(self):
        return {
            'Mode': self.get_mode(),
            'USBprofile': self.get_usb_profile()
        }

    def do_vendor_config(self, config):
        """ Vendor specific configuration (usb mode, sim speed, etc) """
        rst = False
        usb_profile = 'RMNET'
        mode = config.get('ublox', 'mode') or 'Bridge'

        rst |= self.set_usb_profile(usb_profile)
        rst |= self.set_mode(mode)
        rst |= self.disable_secure_data_suite()

        self.enable_smart_temperature_supervisor()

        return rst

    def get_mode(self):
        """ Get function mode (router or bridge) """
        out = self.send_command(b'AT+UIFCONF=2,0,10')

        regex = re.search('UIFCONF:\\s(\\d)', out)
        mode = int(regex.group(1))
        if mode == 0:
            return "Bridge"
        if mode == 1:
            return "Router"
        return "unsupported"

    def set_mode(self, mode):
        """ Returns True if a reset is required """
        actual = self.get_mode()
        if actual != mode:
            if mode == 'Bridge':
                logger.info('Switching modem to bridge mode')
                self.send_command(b'AT+UIFCONF=2,0,10,0', no_answer=True)
                # Save in flash
                self.send_command(b'AT+UIFCONF=2,0,100', no_answer=True)
            elif mode == 'Router':
                logger.info('Switching modem to router mode')
                self.send_command(b'AT+UIFCONF=2,0,10,1', no_answer=True)
                # Save in flash
                self.send_command(b'AT+UIFCONF=2,0,100', no_answer=True)
            else:
                raise Exception('mode not supported')
            return True
        return False

    def get_usb_profile(self):
        """ Get USB profile (only RMNET is supported) """
        try:
            out = self.send_command(b'AT+UUSBCONF?')
        except AtCommandException:
            logger.warning("Empty UUSBCONF: it can happen sometimes")
            out = self.send_command(b'AT+UUSBCONF?')

        regex = re.search('UUSBCONF:\\s\\d,"(.*)",', out)
        return regex.group(1)

    def set_usb_profile(self, profile):
        """ Returns True if a reset is required """
        actual = self.get_usb_profile()
        if actual != profile:
            if profile == 'RMNET':
                logger.info('Switching usb mode to RMNET')
                self.send_command(b'AT+UUSBCONF=4,"RMNET"', no_answer=True)
            else:
                raise Exception('mode not supported')
            return True
        return False

    def set_apn(self, _apn, _username, _password):
        """ Set default APN """
        logger.info("Default APN does not need to be set on LARA-L6: ignoring")

    def disable_secure_data_suite(self):
        """ Disables the secure data suite features of the modem """
        self.send_command(b'AT+CFUN=0', no_answer=True)
        resp = self.send_command(b'AT+USECMODE?')
        regex = re.search(r'\+USECMODE:\s(\d)', resp)
        if regex.group(1) == '0':
            # Already disabled: reenabling connectivity
            self.send_command(b'AT+CFUN=1', no_answer=True)
            return False

        # Disable and ask for reset
        self.send_command(b'AT+USECMODE=0', no_answer=True)
        return True

    def enable_smart_temperature_supervisor(self):
        """
        Smart temperature supervisor will send an URC when the temperature
        is too high or too low and shutdown the modem if it gets worse.
        """
        logger.info("Enabling smart temperature supervisor")
        self.send_command(b'AT+USTS=1', no_answer=True)
