#!/usr/bin/env python3

import serial
import os
import time
import re
import gpiod

from ..exceptions import *
from ..logger import logger
from ..platform import Platform

POWERON_MAX_DELAY_S = 20
POWEROFF_MAX_DELAY_S = 7.5

class WwanModem:
    LATEST_FW = 0
    DEP_FW = 1  # Deprecated: still supported but upgrade possible
    OBS_FW = 2  # Obsolete: not supported anymore
    UNTESTED_FW = 3

    serial = None
    name = None
    pwr = None

    def __init__(self, device_name):
        self.name = device_name
        self.pwr_line = gpiod.find_line('GSM_PWR')
        self.pwr_line.request(consumer=self.name, type=gpiod.LINE_REQ_DIR_OUT, default_vals=[0])

        while True:
            try:
                self.power_up()
                break
            except ModemNotPoweredException as e:
                logger.error("The modem did not power up, turning power off and on again")
                self.power_down()
                # Make sure the power is down long enough to avoid a glitch
                # With toby-l2 this down time must be at least 130ms
                time.sleep(1)

        self.open_serial()

    def get_name(self):
        return self.name

    def get_serial_name(self):
        return '/dev/{}'.format(self.name)

    def send_command(self, cmd, no_answer=False):
        retry = False
        while True:
            self.serial.flushInput()
            self.serial.write(cmd)
            self.serial.write(b'\r')
            s = self.serial.read_until(b'OK')
            if(len(s.decode('utf-8')) < 4):
                # Avoid empty response just after reset
                continue
            if b'ERROR' in s:
                raise AtCommandReturnedError()
            elif not b'OK' in s and retry:
                raise AtCommandException("Unhandled answer from modem : {}".format(s))
            elif not b'OK' in s and not retry:
                logger.warn("Modem did not answer properly, retrying command")
                # Did not receive a proper response
                retry = True
                continue
            else:
                # Proper answer
                break

        if no_answer:
            return

        x = re.search('(.*)\n.*\nOK', s.decode('utf-8').replace('\r', ''))
        if not x:
            logger.error("answer from modem not as expected: \n{}".format(s.decode('utf-8')))
            raise AtCommandException("Modem answer not parsed properly")
        return x.group(1)

    def power_down(self):
        logger.info("modem power down")

        if not Platform.is_pwr_controllable(self.name):
            logger.info("The power cannot be controlled, not shutting down");
            raise PwrNotControllableException

        if not os.path.exists(self.get_serial_name()):
            logger.warn("The modem is already off. Making sure that power is cut")
            self.set_pwr_state(0)
            return

        if self.serial is None:
            logger.warning("The modem is on but the serial was not open.")
            logger.warning("Is the power-on timeout too small ?")
            self.set_pwr_state(0)
            self.wait_on_down()
            return

        self.serial.write(b'AT+CPWROFF\r')

        # Wait until modem voltage is 0 if available
        timeout_s = POWEROFF_MAX_DELAY_S
        try:
            sleep_time = 0.5
            while Platform.get_modem_voltage_mV(self.name) != 0:
                time.sleep(sleep_time)
                timeout_s -= sleep_time
                if timeout_s <= 0:
                    logger.error("Timeout: modem did not turn off by itself")
                    break
            logger.info("Modem voltage reached 0")

        except VoltageNotAvailableException:
            logger.info("Using default poweroff delay")
            time.sleep(POWEROFF_MAX_DELAY_S)

        self.set_pwr_state(0)
        self.wait_on_down()

    def get_vendor(self):
        return self.send_command(b'AT+CGMI')

    def get_model(self):
        return self.send_command(b'AT+CGMM')

    def get_firmware_version(self):
        return self.send_command(b'ATI9')

    def get_sim_id(self):
        try:
            return self.send_command(b'AT+CCID?')
        except AtCommandReturnedError:
            raise NoSimException()


    def close_serial(self):
        self.serial.close()

    def open_serial(self):
        serial_name = self.get_serial_name()
        if os.path.exists(serial_name):
            self.serial = serial.Serial(serial_name, 115200, timeout=2)
        else:
            raise Exception("Serial device does not exist")

    def wait_on_down(self):
        while os.path.exists(self.get_serial_name()):
            time.sleep(1)

    def wait_on_up(self):
        timeout_s = POWERON_MAX_DELAY_S;
        while not os.path.exists(self.get_serial_name()):
            time.sleep(1)
            timeout_s -= 1
            if timeout_s == 0:
                raise ModemNotPoweredException()

    def set_pwr_state(self, value):
        state = "enabled" if value == 1 else "disabled"
        with open("/sys/devices/platform/" + self.name + "-supply/state", "w") as supply_file:
            supply_file.write(state + "\n")

    def power_up(self):
        logger.info("modem power up")

        if Platform.is_pwr_controllable(self.name):
            self.set_pwr_state(1)
            time.sleep(1)

        # Send pulse on power line
        self.pwr_line.set_value(1)
        time.sleep(0.3)
        self.pwr_line.set_value(0)

        self.wait_on_up()

    # Return True if reset is required
    def do_vendor_config(self, config):
        # Placeholder to be implemented if required
        return False

    # Return a dictionary with vendor configurations
    def get_vendor_configs(self):
        # Placeholder to be implemented if required
        return {}



def main():
    modem = WwanModem('wwan0')
    print(modem.get_vendor())
    print(modem.get_model())
    print(modem.get_firmware_version())
    print(modem.get_sim_id())


if __name__ == '__main__':
    main()
