import json
import subprocess
from .exceptions import VoltageNotAvailableException
from .sim_selectors.none import SimSelectorNone
from .sim_selectors.single_switch import SimSelectorSingleSwitch

DT_PATH = '/proc/device-tree/'


def _read_from_dt(modem, filename):
    with open(DT_PATH + modem + '/' + filename, 'r') as f:
        return f.read().rstrip('\0')

def _get_gpio_map(modem='wwan0'):
    sim_map = dict()
    low_sims_names = _read_from_dt(modem, 'low-sims').split('\0')
    high_sims_names = _read_from_dt(modem, 'high-sims').split('\0')
    for sim in low_sims_names:
        sim_map[sim] = 0
    for sim in high_sims_names:
        sim_map[sim] = 1

    return sim_map


class Platform:

    @staticmethod
    def get_default_sim():
        return "sim1"

    @staticmethod
    def get_sim_selector(modem='wwan0'):
        try:
            sim_select_type = _read_from_dt(modem, 'sim-selector')
        except FileNotFoundError:
            sim_select_type = "none"

        if sim_select_type == "gpio":
            sim_map = _get_gpio_map(modem)
            return SimSelectorSingleSwitch(sim_map, Platform.get_default_sim())
        elif sim_select_type == "none":
            return SimSelectorNone()
        else:
            raise Exception("Unsupported type of sim selector")

    @staticmethod
    def is_pwr_controllable(modem):
        try:
            _read_from_dt(modem, 'pwr-always-on')
            return False
        except FileNotFoundError:
            return True

    @staticmethod
    def get_modem_voltage_mV(modem): # pylint: disable=C0103
        """ Returns modem voltage in mV if available or throws an error """
        line = subprocess.run(['sensors', '-j', '-A'], stdout=subprocess.PIPE).stdout.decode('utf-8')
        sensors = json.loads(line)
        for sensor in sensors.keys():
            try:
                supply = sensors[sensor][f"{modem}-supply"]
            except KeyError:
                continue
            if supply is not None:
                driver_label = list(supply.keys())[0]
                voltage = supply[driver_label]
                return int(float(voltage) * 1000)

        raise VoltageNotAvailableException
