#!/usr/bin/env python3

import os
import time
import sys
import signal

from .wwan_config_manager import WwanConfigManager
from .config_migration import migrate_config
from .modem_manager import ModemManager, ModemManagerException
from .logger import logger
from .exceptions import PwrNotControllableException

class Daemon():
    def __init__(self, config_manager):
        self.config_manager = config_manager
        self.monitoring_inhibited = False
        self.monitor = ModemManager()

    def stop_monitoring(self):
        self.monitoring_inhibited = True

    def start_monitoring(self):
        self.monitoring_inhibited = False

    def stop(self):
        self.stop_monitoring()
        self.config_manager.stop_modem_manager()
        try:
            self.config_manager.stop_modem()
        except PwrNotControllableException:
            logger.info("Cannot stop the modem, so switch back the sim to default")
            self.config_manager.reconnect_default_sim()
        sys.exit(0)

    def signal_handler(self, sig):
        """ Request form other tools to access the modem directly (eg. firmware update) """

        # This signal handler is spawning new processes to control MM service
        # This may lead to a problem with systemd default KillMode (control-group),
        # killing the subprocesses. In order to avoid this, the systemd service must set
        # KillMode=mixed

        if sig == signal.SIGTERM:
            self.stop()
        elif sig == signal.SIGUSR1:
            logger.info("MM monitoring inhibited")
            self.stop_monitoring()
            self.config_manager.stop_modem_manager()
        else:
            logger.info("MM monitoring restart")
            self.config_manager.start_modem_manager()
            self.start_monitoring()


    def loop(self):
        while True:
            time.sleep(10)

            if self.monitoring_inhibited:
                continue

            try:
                self.monitor.monitor()
            except ModemManagerException:
                logger.warning("ModemManager cannot recover modem, doing power cycle")
                self.config_manager.recover_modem()


def fork(device_name):
    pid = os.fork()
    if pid > 0:
        with open('/run/{}.pid'.format(device_name), 'w') as pidfile:
            pidfile.write(str(pid))
            sys.exit(0)

def daemonize(config_manager):

    daemon = Daemon(config_manager)

    def signal_handler(sig, _frame):
        daemon.signal_handler(sig)

    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGUSR1, signal_handler)
    signal.signal(signal.SIGUSR2, signal_handler)

    daemon.loop()

def main():
    if len(sys.argv) == 2 and sys.argv[1].startswith('wwan'):
        device_name = sys.argv[1]
        migrate_config(device_name)
        mgr = WwanConfigManager(device_name)

        fork(device_name)
        daemonize(mgr)

    else:
        print("Usage: wwan-config <modem name>")
        sys.exit(1)


if __name__ == '__main__':
    main()
