#!/usr/bin/env python3

import os
import time
import configparser

from .platform import Platform
from .modems.wwan_modem import WwanModem
from .modems.ublox.base import UbloxWwanModem
from .modems.ublox.toby import UbloxTobyWwanModem
from .modems.ublox.lara import UbloxLaraWwanModem
from .modem_manager import ModemManager, ModemManagerException
from .logger import logger

CONF_PATH = '/run/wwan/'
SETUP_PATH = '/etc/wwan/'


def _write_config_file(file, label, value):
    line = (label + ':').ljust(20) + value + '\n'
    file.write(line)


def _dump_firmware(config_file, fw):
    version, status = fw
    if status == WwanModem.OBS_FW:
        config = '{} (Obsolete)'.format(version)
    elif status == WwanModem.DEP_FW:
        config = '{} (Deprecated)'.format(version)
    elif status == WwanModem.UNTESTED_FW:
        config = '{} (Untested)'.format(version)
    else:
        config = version
    _write_config_file(config_file, 'Firmware', config)




class WwanConfigManager():

    def __init__(self, modem_name):
        self.config = configparser.ConfigParser()
        self.config.read(SETUP_PATH + modem_name + '.conf')

        # Makes sure ModemManager is stopped
        self.stop_modem_manager()

        # Power on and initialize modem
        self.modem = WwanModem(modem_name)
        if self.modem.get_vendor() == 'u-blox':
            logger.info("u-blox modem")
            if self.modem.get_model() == "TOBY-L210":
                logger.info("TOBY-L210 modem")
                self.modem = UbloxTobyWwanModem(self.modem)
            elif self.modem.get_model().startswith("LARA-L6"):
                logger.info("LARA-L6 modem")
                self.modem = UbloxLaraWwanModem(self.modem)
            else:
                self.modem = UbloxWwanModem(self.modem)

        # Setup sim cards
        self.sim_selector = Platform.get_sim_selector()
        sim_reload = self.setup_sim()
        if sim_reload:
            self.modem.reload_sim()

        self.configure_modem()
        self.modem.close_serial()

        self.start_modem_manager()

    def setup_sim(self):
        """ Return True if sim must be reloaded by the modem """
        sim0 = self.config.get('sim', 'SIM') or 'auto'
        if sim0 == 'auto':
            # Switch to eSIM and reset if main is not available
            return self.sim_selector.auto_connect(self.modem)
        else:
            return self.sim_selector.connect(sim0)

    def reconnect_default_sim(self):
        if self.sim_selector.connect(Platform.get_default_sim()):
            self.modem.open_serial()
            self.modem.reload_sim()
            self.modem.close_serial()

    def dump_config(self):
        logger.info("Writing modem config and status to /run")
        os.makedirs(CONF_PATH, exist_ok=True)

        with open(CONF_PATH + self.modem.get_name() + '.config', 'w') as config_file:
            vendor = self.modem.get_vendor()
            _write_config_file(config_file, 'Vendor', vendor)
            _write_config_file(config_file, 'Model', self.modem.get_model())
            _dump_firmware(config_file, self.modem.get_firmware())

            vendor_configs = self.modem.get_vendor_configs()
            for name, value in vendor_configs.items():
                _write_config_file(config_file, name, value)


    def configure_modem(self):
        rst = False

        # Vendor specific configuration
        rst |= self.modem.do_vendor_config(self.config)

        # Reset if required
        if rst:
            logger.info("reset required because of vendor configs")
            self.modem.reset()

        # Setup APN
        apn = self.config.get('apn', 'apn') or None
        if apn:
            user = self.config.get('apn', 'user') or None
            password = self.config.get('apn', 'password') or None
            self.modem.set_apn(apn, user, password)

        # Write setup to /run
        self.dump_config()



    def start_modem_manager(self):
        while True:
            try:
                ModemManager.monitor_startup()
                break
            except ModemManagerException:
                self.recover_modem()

    def stop_modem_manager(self):
        ModemManager.stop()

    def stop_modem(self):
        self.modem.open_serial()
        try:
            self.modem.power_down()
        finally:
            self.modem.close_serial()

    def start_modem(self):
        self.modem.power_up()
        self.modem.open_serial()
        self.configure_modem()
        self.modem.close_serial()

    def recover_modem(self):
        self.stop_modem()
        time.sleep(5)
        self.start_modem()
